@@ -78,7 +78,10 @@ xsltXPathFunctionLookup (xmlXPathContextPtr ctxt,
 #endif
 
     /* give priority to context-level functions */
+    /*
     ret = (xmlXPathFunction) xmlHashLookup2(ctxt->funcHash, name, ns_uri);
+    */
+    XML_CAST_FPTR(ret) = xmlHashLookup2(ctxt->funcHash, name, ns_uri);
 
     if (ret == NULL)
 	ret = xsltExtModuleFunctionLookup(name, ns_uri);
@@ -104,23 +107,23 @@ xsltDocumentFunctionLoadDocument(xmlXPathParserContextPtr ctxt, xmlChar* URI)
     xsltTransformContextPtr tctxt;
     xmlURIPtr uri;
     xmlChar *fragment;
-    xsltDocumentPtr xsltdoc;
+    xsltDocumentPtr idoc; /* document info */
     xmlDocPtr doc;
     xmlXPathContextPtr xptrctxt = NULL;
-    xmlXPathObjectPtr object = NULL;
+    xmlXPathObjectPtr resObj = NULL;
 
     tctxt = xsltXPathGetTransformContext(ctxt);
     if (tctxt == NULL) {
-	xsltTransformError(xsltXPathGetTransformContext(ctxt), NULL, NULL,
-			   "document() : internal error tctxt == NULL\n");
+	xsltTransformError(NULL, NULL, NULL,
+	    "document() : internal error tctxt == NULL\n");
 	valuePush(ctxt, xmlXPathNewNodeSet(NULL));
 	return;
     } 
 	
     uri = xmlParseURI((const char *) URI);
     if (uri == NULL) {
-	xsltTransformError(xsltXPathGetTransformContext(ctxt), NULL, NULL,
-			   "document() : failed to parse URI\n");
+	xsltTransformError(tctxt, NULL, NULL,
+	    "document() : failed to parse URI\n");
 	valuePush(ctxt, xmlXPathNewNodeSet(NULL));
 	return;
     } 
@@ -132,16 +135,20 @@ xsltDocumentFunctionLoadDocument(xmlXPathParserContextPtr ctxt, xmlChar* URI)
     if (fragment != NULL) {
 	uri->fragment = NULL;
 	URI = xmlSaveUri(uri);
-	xsltdoc = xsltLoadDocument(tctxt, URI);
+	idoc = xsltLoadDocument(tctxt, URI);
 	xmlFree(URI);
     } else
-	xsltdoc = xsltLoadDocument(tctxt, URI);
+	idoc = xsltLoadDocument(tctxt, URI);
     xmlFreeURI(uri);
     
-    if (xsltdoc == NULL) {
+    if (idoc == NULL) {
 	if ((URI == NULL) ||
 	    (URI[0] == '#') ||
-	    (xmlStrEqual(tctxt->style->doc->URL, URI))) {
+	    (xmlStrEqual(tctxt->style->doc->URL, URI))) 
+	{
+	    /*
+	    * This selects the stylesheet's doc itself.
+	    */
 	    doc = tctxt->style->doc;
 	} else {
 	    valuePush(ctxt, xmlXPathNewNodeSet(NULL));
@@ -152,11 +159,10 @@ xsltDocumentFunctionLoadDocument(xmlXPathParserContextPtr ctxt, xmlChar* URI)
 	    return;
 	}
     } else
-	doc = xsltdoc->doc;
+	doc = idoc->doc;
 
-    if ( fragment == NULL ) {
-	valuePush(ctxt,
-		  xmlXPathNewNodeSet((xmlNodePtr) doc));
+    if (fragment == NULL) {
+	valuePush(ctxt, xmlXPathNewNodeSet((xmlNodePtr) doc));
 	return;
     }
 	
@@ -164,43 +170,42 @@ xsltDocumentFunctionLoadDocument(xmlXPathParserContextPtr ctxt, xmlChar* URI)
 #ifdef LIBXML_XPTR_ENABLED
     xptrctxt = xmlXPtrNewContext(doc, NULL, NULL);
     if (xptrctxt == NULL) {
-	xsltTransformError(xsltXPathGetTransformContext(ctxt), NULL, NULL,
-			   "document() : internal error xptrctxt == NULL\n");
+	xsltTransformError(tctxt, NULL, NULL,
+	    "document() : internal error xptrctxt == NULL\n");
 	goto out_fragment;
     }
 
-    object = xmlXPtrEval(fragment, xptrctxt);
+    resObj = xmlXPtrEval(fragment, xptrctxt);
+    xmlXPathFreeContext(xptrctxt);
 #endif
-    xmlFree(fragment);
-	if (xptrctxt != NULL)
-    		xmlXPathFreeContext(xptrctxt);
+    xmlFree(fragment);	
 
-    if (object == NULL)
+    if (resObj == NULL)
 	goto out_fragment;
 	
-    switch (object->type) {
-    case XPATH_NODESET:
-	break;
-    case XPATH_UNDEFINED:
-    case XPATH_BOOLEAN:
-    case XPATH_NUMBER:
-    case XPATH_STRING:
-    case XPATH_POINT:
-    case XPATH_USERS:
-    case XPATH_XSLT_TREE:
-    case XPATH_RANGE:
-    case XPATH_LOCATIONSET:
-	xsltTransformError(xsltXPathGetTransformContext(ctxt), NULL, NULL,
-			   "document() : XPointer does not select a node set: #%s\n", 
-			   fragment);
+    switch (resObj->type) {
+	case XPATH_NODESET:
+	    break;
+	case XPATH_UNDEFINED:
+	case XPATH_BOOLEAN:
+	case XPATH_NUMBER:
+	case XPATH_STRING:
+	case XPATH_POINT:
+	case XPATH_USERS:
+	case XPATH_XSLT_TREE:
+	case XPATH_RANGE:
+	case XPATH_LOCATIONSET:
+	    xsltTransformError(tctxt, NULL, NULL,
+		"document() : XPointer does not select a node set: #%s\n", 
+		fragment);
 	goto out_object;
     }
     
-    valuePush(ctxt, object);
+    valuePush(ctxt, resObj);
     return;
 
 out_object:
-    xmlXPathFreeObject(object);
+    xmlXPathFreeObject(resObj);
 
 out_fragment:
     valuePush(ctxt, xmlXPathNewNodeSet(NULL));
@@ -342,22 +347,19 @@ xsltDocumentFunction(xmlXPathParserContextPtr ctxt, int nargs)
  *   node-set key(string, object)
  */
 void
-xsltKeyFunction(xmlXPathParserContextPtr ctxt, int nargs){
-    xmlNodeSetPtr nodelist;
-    xmlXPathObjectPtr obj1, obj2;
-    xmlChar *key = NULL, *value;
-    const xmlChar *keyURI;
-    xsltTransformContextPtr tctxt;
-    xsltDocumentPtr oldDocumentPtr;
-    xmlDocPtr oldXPathDocPtr;
+xsltKeyFunction(xmlXPathParserContextPtr ctxt, int nargs){    
+    xmlXPathObjectPtr obj1, obj2;    
     
     if (nargs != 2) {
 	xsltTransformError(xsltXPathGetTransformContext(ctxt), NULL, NULL,
 		"key() : expects two arguments\n");
 	ctxt->error = XPATH_INVALID_ARITY;
 	return;
-    }
+    }    
 
+    /*
+    * Get the key's value.
+    */
     obj2 = valuePop(ctxt);
     xmlXPathStringFunction(ctxt, 1);
     if ((obj2 == NULL) ||
@@ -369,7 +371,10 @@ xsltKeyFunction(xmlXPathParserContextPtr ctxt, int nargs){
 
 	return;
     }
-    obj1 = valuePop(ctxt);
+    /*
+    * Get the key's name.
+    */
+    obj1 = valuePop(ctxt);    
 
     if ((obj2->type == XPATH_NODESET) || (obj2->type == XPATH_XSLT_TREE)) {
 	int i;
@@ -392,8 +397,26 @@ xsltKeyFunction(xmlXPathParserContextPtr ctxt, int nargs){
 	}
 	valuePush(ctxt, ret);
     } else {
+	xmlNodeSetPtr nodelist = NULL;
+	xmlChar *key = NULL, *value;
+	const xmlChar *keyURI;
+	xsltTransformContextPtr tctxt;   
 	xmlChar *qname, *prefix;
+	xmlXPathContextPtr xpctxt = ctxt->context;
+	xmlNodePtr tmpNode = NULL;
+	xsltDocumentPtr oldDocInfo;
 
+	tctxt = xsltXPathGetTransformContext(ctxt);
+
+	oldDocInfo = tctxt->document;
+
+	if (xpctxt->node == NULL) {
+	    xsltTransformError(tctxt, NULL, tctxt->inst,
+		"Internal error in xsltKeyFunction(): "
+		"The context node is not set on the XPath context.\n");
+	    tctxt->state = XSLT_STATE_STOPPED;
+	    goto error;
+	}	
 	/*
 	 * Get the associated namespace URI if qualified name
 	 */
@@ -406,10 +429,13 @@ xsltKeyFunction(xmlXPathParserContextPtr ctxt, int nargs){
 		xmlFree(prefix);
 	} else {
 	    if (prefix != NULL) {
-		keyURI = xmlXPathNsLookup(ctxt->context, prefix);
+		keyURI = xmlXPathNsLookup(xpctxt, prefix);
 		if (keyURI == NULL) {
-		    xsltTransformError(xsltXPathGetTransformContext(ctxt), NULL, NULL,
+		    xsltTransformError(tctxt, NULL, tctxt->inst,
 			"key() : prefix %s is not bound\n", prefix);
+		    /*
+		    * TODO: Shouldn't we stop here?
+		    */
 		}
 		xmlFree(prefix);
 	    } else {
@@ -423,59 +449,90 @@ xsltKeyFunction(xmlXPathParserContextPtr ctxt, int nargs){
 	valuePush(ctxt, obj2);
 	xmlXPathStringFunction(ctxt, 1);
 	if ((ctxt->value == NULL) || (ctxt->value->type != XPATH_STRING)) {
-	    xsltTransformError(xsltXPathGetTransformContext(ctxt), NULL, NULL,
+	    xsltTransformError(tctxt, NULL, tctxt->inst,
 		"key() : invalid arg expecting a string\n");
 	    ctxt->error = XPATH_INVALID_TYPE;
-	    xmlXPathFreeObject(obj1);
-
-	    return;
+	    goto error;
 	}
 	obj2 = valuePop(ctxt);
 	value = obj2->stringval;
-
-	tctxt = xsltXPathGetTransformContext(ctxt);
-	oldDocumentPtr = tctxt->document;
-	oldXPathDocPtr = tctxt->xpathCtxt->doc;
-	if ((ctxt->context->doc != NULL) &&
-		    (tctxt->document->doc != ctxt->context->doc)) {
+	
+	/*
+	* We need to ensure that ctxt->document is available for
+	* xsltGetKey().
+	* First find the relevant doc, which is the context node's
+	* owner doc; using context->doc is not safe, since
+	* the doc could have been acquired via the document() function,
+	* or the doc might be a Result Tree Fragment.
+	* FUTURE INFO: In XSLT 2.0 the key() function takes an additional
+	* argument indicating the doc to use.
+	*/	
+	if (xpctxt->node->type == XML_NAMESPACE_DECL) {
 	    /*
-	     * The xpath context document needs to be changed.  If the
-	     * current context document is a node-set, we must use an
-	     * xsltDocument associated with the node-set, which may or
-	     * may not currently exist.
-	     */
-	    if (xmlStrEqual((const xmlChar *)ctxt->context->doc->name,
-	    		BAD_CAST " fake node libxslt")) {	/* node-set */
-		/*
-		 * Check whether we already have an xsltDocument set up
-		 */
-		if (ctxt->context->doc->_private == NULL)	/* nope */
-		    ctxt->context->doc->_private =
-		    	xsltNewDocument(tctxt, ctxt->context->doc);
-	        tctxt->document = ctxt->context->doc->_private;
+	    * REVISIT: This is a libxml hack! Check xpath.c for details.
+	    * The XPath module sets the owner element of a ns-node on
+	    * the ns->next field.
+	    */
+	    if ((((xmlNsPtr) xpctxt->node)->next != NULL) &&
+		(((xmlNsPtr) xpctxt->node)->next->type == XML_ELEMENT_NODE))
+	    {
+		tmpNode = (xmlNodePtr) ((xmlNsPtr) xpctxt->node)->next;
 	    }
-	    else {
-	        tctxt->document = xsltFindDocument(tctxt, ctxt->context->doc);
-	        if (tctxt->document == NULL)
-	            tctxt->document = oldDocumentPtr;
-	        else
-	            tctxt->xpathCtxt->doc = ctxt->context->doc;
+	} else
+	    tmpNode = xpctxt->node;
+
+	if ((tmpNode == NULL) || (tmpNode->doc == NULL)) {
+	    xsltTransformError(tctxt, NULL, tctxt->inst,
+		"Internal error in xsltKeyFunction(): "
+		"Couldn't get the doc of the XPath context node.\n");
+	    goto error;
+	}
+
+	if ((tctxt->document == NULL) ||
+	    (tctxt->document->doc != tmpNode->doc))
+	{	   
+	    if (tmpNode->doc->name && (tmpNode->doc->name[0] == ' ')) {
+		/*
+		* This is a Result Tree Fragment.
+		*/
+		if (tmpNode->doc->_private == NULL) {
+		    tmpNode->doc->_private = xsltNewDocument(tctxt, tmpNode->doc);
+		    if (tmpNode->doc->_private == NULL)
+			goto error;
+		}
+		tctxt->document = (xsltDocumentPtr) tmpNode->doc->_private;		
+	    } else {
+		/*
+		* May be the initial source doc or a doc acquired via the
+		* document() function.
+		*/
+		tctxt->document = xsltFindDocument(tctxt, tmpNode->doc);
+	    }
+	    if (tctxt->document == NULL) {
+		xsltTransformError(tctxt, NULL, tctxt->inst,
+		    "Internal error in xsltKeyFunction(): "
+		    "Could not get the document info of a context doc.\n");
+		tctxt->state = XSLT_STATE_STOPPED;
+		goto error;
 	    }
 	}
+	/*
+	* Get/compute the key value.
+	*/
 	nodelist = xsltGetKey(tctxt, key, keyURI, value);
-	tctxt->document = oldDocumentPtr;
-	tctxt->xpathCtxt->doc = oldXPathDocPtr;
-	valuePush(ctxt, xmlXPathWrapNodeSet(
-		        xmlXPathNodeSetMerge(NULL, nodelist)));
-    }
 
+error:	
+	tctxt->document = oldDocInfo;
+	valuePush(ctxt, xmlXPathWrapNodeSet(
+	    xmlXPathNodeSetMerge(NULL, nodelist)));
+	if (key != NULL)
+	    xmlFree(key);
+    }    
 
     if (obj1 != NULL)
 	xmlXPathFreeObject(obj1);
     if (obj2 != NULL)
-	xmlXPathFreeObject(obj2);
-    if (key != NULL)
-	xmlFree(key);
+	xmlXPathFreeObject(obj2);    
 }
 
 /**
