@@ -27,32 +27,34 @@
 extern "C" {
 #endif
 
+/* #define XSLT_DEBUG_PROFILE_CACHE */
+
+/**
+ * XSLT_IS_TEXT_NODE:
+ *
+ * check if the argument is a text node
+ */
 #define XSLT_IS_TEXT_NODE(n) ((n != NULL) && \
     (((n)->type == XML_TEXT_NODE) || \
      ((n)->type == XML_CDATA_SECTION_NODE)))
 
 
-#if 0
-
-extern const xmlChar *xsltDocFragFake;
-
-#define XSLT_MARK_RES_TREE_FRAG(n) (n)->psvi = (void *) xsltDocFragFake;
-
-#define XSLT_IS_RES_TREE_FRAG(n) \
-    ((n != NULL) && ((n)->type == XML_DOCUMENT_NODE) && \
-     ((n)->psvi == xsltDocFragFake))
-
-#else
-
+/**
+ * XSLT_MARK_RES_TREE_FRAG:
+ *
+ * internal macro to set up tree fragments
+ */
 #define XSLT_MARK_RES_TREE_FRAG(n) \
     (n)->name = (char *) xmlStrdup(BAD_CAST " fake node libxslt");
 
+/**
+ * XSLT_IS_RES_TREE_FRAG:
+ *
+ * internal macro to test tree fragments
+ */
 #define XSLT_IS_RES_TREE_FRAG(n) \
     ((n != NULL) && ((n)->type == XML_DOCUMENT_NODE) && \
-     ((n)->name != NULL) && ((n)->name[0] == ' ') && \
-    xmlStrEqual(BAD_CAST (n)->name, BAD_CAST " fake node libxslt"))
-
-#endif
+     ((n)->name != NULL) && ((n)->name[0] == ' '))
 
 /**
  * XSLT_REFACTORED_KEYCOMP:
@@ -61,6 +63,14 @@ extern const xmlChar *xsltDocFragFake;
  */
 #define XSLT_REFACTORED_KEYCOMP
 
+/**
+ * XSLT_FAST_IF:
+ *
+ * Internal define to enable usage of xmlXPathCompiledEvalToBoolean()
+ * for XSLT "tests"; e.g. in <xsl:if test="/foo/bar">
+ */
+#define XSLT_FAST_IF
+
 /**
  * XSLT_REFACTORED:
  *
@@ -69,7 +79,18 @@ extern const xmlChar *xsltDocFragFake;
 /* #define XSLT_REFACTORED */
 /* ==================================================================== */
 
+/**
+ * XSLT_REFACTORED_VARS:
+ *
+ * Internal define to enable the refactored variable part of libxslt
+ */
+#define XSLT_REFACTORED_VARS
+
 #ifdef XSLT_REFACTORED
+
+extern const xmlChar *xsltXSLTAttrMarker;
+
+
 /* TODO: REMOVE: #define XSLT_REFACTORED_EXCLRESNS */
 
 /* TODO: REMOVE: #define XSLT_REFACTORED_NSALIAS */
@@ -80,7 +101,7 @@ extern const xmlChar *xsltDocFragFake;
  * Internal define to enable the pointer-comparison of
  * namespaces of XSLT elements. 
  */
-#define XSLT_REFACTORED_XSLT_NSCOMP
+/* #define XSLT_REFACTORED_XSLT_NSCOMP */
 
 /**
  * XSLT_REFACTORED_XPATHCOMP
@@ -94,10 +115,20 @@ extern const xmlChar *xsltDocFragFake;
 
 extern const xmlChar *xsltConstNamespaceNameXSLT;
 
+/**
+ * IS_XSLT_ELEM_FAST:
+ *
+ * quick test to detect XSLT elements
+ */
 #define IS_XSLT_ELEM_FAST(n) \
     (((n) != NULL) && ((n)->ns != NULL) && \
     ((n)->ns->href == xsltConstNamespaceNameXSLT))
 
+/**
+ * IS_XSLT_ATTR_FAST:
+ *
+ * quick test to detect XSLT attributes
+ */
 #define IS_XSLT_ATTR_FAST(a) \
     (((a) != NULL) && ((a)->ns != NULL) && \
     ((a)->ns->href == xsltConstNamespaceNameXSLT))
@@ -240,6 +271,7 @@ struct _xsltTemplate {
     /* Profiling informations */
     int nbCalls;        /* the number of time the template was called */
     unsigned long time; /* the time spent in this template */
+    void *params;       /* xsl:param instructions */
 };
 
 /**
@@ -272,7 +304,6 @@ struct _xsltDecimalFormat {
  *
  * Data structure associated to a parsed document.
  */
-
 typedef struct _xsltDocument xsltDocument;
 typedef xsltDocument *xsltDocumentPtr;
 struct _xsltDocument {
@@ -388,15 +419,16 @@ typedef enum {
     XSLT_FUNC_PARAM,
     XSLT_FUNC_VARIABLE,
     XSLT_FUNC_WHEN,
-    XSLT_FUNC_EXTENSION,
+    XSLT_FUNC_EXTENSION
 #ifdef XSLT_REFACTORED
+    ,
     XSLT_FUNC_OTHERWISE,
     XSLT_FUNC_FALLBACK,
     XSLT_FUNC_MESSAGE,
     XSLT_FUNC_INCLUDE,
     XSLT_FUNC_ATTRSET,
     XSLT_FUNC_LITERAL_RESULT_ELEMENT,
-    XSLT_FUNC_UNKOWN_FORWARDS_COMPAT,
+    XSLT_FUNC_UNKOWN_FORWARDS_COMPAT
 #endif
 } xsltStyleType;
 
@@ -466,7 +498,8 @@ typedef struct _xsltNsListContainer xsltNsListContainer;
 typedef xsltNsListContainer *xsltNsListContainerPtr;
 struct _xsltNsListContainer {
     xmlNsPtr *list;
-    int number;
+    int totalNumber;
+    int xpathNumber;    
 };
 
 /**
@@ -474,6 +507,8 @@ struct _xsltNsListContainer {
  * 
  * Fields for API compatibility to the structure
  * _xsltElemPreComp which is used for extension functions.
+ * Note that @next is used for storage; it does not reflect a next
+ * sibling in the tree.
  * TODO: Evaluate if we really need such a compatibility.
  */
 #define XSLT_ITEM_COMPATIBILITY_FIELDS \
@@ -590,13 +625,13 @@ typedef xsltStyleItemElement *xsltStyleItemElementPtr;
 struct _xsltStyleItemElement {
     XSLT_ITEM_COMMON_FIELDS 
 
-    const xmlChar *use;		/* copy, element */
-    int      has_use;		/* copy, element */
-    const xmlChar *name;	/* element, attribute, pi */
-    int      has_name;		/* element, attribute, pi */
-    const xmlChar *ns;		/* element */
-    int      has_ns;		/* element */
-
+    const xmlChar *use;
+    int      has_use;
+    const xmlChar *name;    
+    int      has_name;
+    const xmlChar *ns;
+    const xmlChar *nsPrefix;
+    int      has_ns;
 };
 
 /**
@@ -614,10 +649,11 @@ typedef xsltStyleItemAttribute *xsltStyleItemAttributePtr;
 
 struct _xsltStyleItemAttribute {
     XSLT_ITEM_COMMON_FIELDS
-    const xmlChar *name;	/* element, attribute, pi */
-    int      has_name;		/* element, attribute, pi */
-    const xmlChar *ns;		/* element  attribute */
-    int      has_ns;		/* element  attribute */
+    const xmlChar *name;
+    int      has_name;
+    const xmlChar *ns;
+    const xmlChar *nsPrefix;
+    int      has_ns;
 };
 
 /**
@@ -689,7 +725,7 @@ typedef struct _xsltStyleItemApplyTemplates xsltStyleItemApplyTemplates;
 typedef xsltStyleItemApplyTemplates *xsltStyleItemApplyTemplatesPtr;
 
 struct _xsltStyleItemApplyTemplates {
-   XSLT_ITEM_COMMON_FIELDS
+    XSLT_ITEM_COMMON_FIELDS
 
     const xmlChar *mode;	/* apply-templates */
     const xmlChar *modeURI;	/* apply-templates */
@@ -935,9 +971,21 @@ typedef xsltStyleItemVariable *xsltStyleItemVariablePtr;
  *   <!-- Content: template -->
  * </xsl:param>
  */
-typedef xsltStyleBasicItemVariable xsltStyleItemParam;
+typedef struct _xsltStyleItemParam xsltStyleItemParam;
 typedef xsltStyleItemParam *xsltStyleItemParamPtr;
 
+struct _xsltStyleItemParam {
+    XSLT_ITEM_COMMON_FIELDS
+
+    const xmlChar *select;
+    xmlXPathCompExprPtr comp;
+
+    const xmlChar *name;
+    int      has_name;
+    const xmlChar *ns;
+    int      has_ns;    
+};
+
 /**
  * xsltStyleItemWithParam:
  *
@@ -1162,6 +1210,21 @@ struct _xsltNsList {
     xmlNsPtr ns;
 };
 
+/*
+* xsltVarInfo:
+*
+* Used at compilation time for parameters and variables.
+*/
+typedef struct _xsltVarInfo xsltVarInfo;
+typedef xsltVarInfo *xsltVarInfoPtr;
+struct _xsltVarInfo {
+    xsltVarInfoPtr next; /* next in the list */
+    xsltVarInfoPtr prev;
+    int depth; /* the depth in the tree */
+    const xmlChar *name;
+    const xmlChar *nsName;
+};
+
 #define XSLT_ELEMENT_CATEGORY_XSLT 0
 #define XSLT_ELEMENT_CATEGORY_EXTENSION 1
 #define XSLT_ELEMENT_CATEGORY_LRE 2
@@ -1189,6 +1252,7 @@ struct _xsltCompilerNodeInfo {
     xsltPointerListPtr exclResultNs; 
     /* The current extension instruction namespaces */
     xsltPointerListPtr extElemNs;
+
     /* The current info for literal result elements. */
     xsltStyleItemLRElementInfoPtr litResElemInfo;
     /* 
@@ -1257,7 +1321,9 @@ struct _xsltCompilerCtxt {
 #endif
     xsltStyleItemUknownPtr unknownItem;
     int hasNsAliases; /* Indicator if there was an xsl:namespace-alias. */
-    xsltNsAliasPtr nsAliases;    
+    xsltNsAliasPtr nsAliases;
+    xsltVarInfoPtr ivars; /* Storage of local in-scope variables/params. */
+    xsltVarInfoPtr ivar; /* topmost local variable/param. */
 };   
 
 #else /* XSLT_REFACTORED */
@@ -1324,6 +1390,10 @@ struct _xsltStylePreComp {
 
 #endif /* XSLT_REFACTORED */
 
+
+#define XSLT_VAR_GLOBAL 1<<0
+#define XSLT_VAR_IN_SELECT 1<<1
+#define XSLT_TCTXT_VARIABLE(c) ((xsltStackElemPtr) (c)->contextVariable)
 /*
  * The in-memory structure corresponding to an XSLT Variable
  * or Param.
@@ -1337,8 +1407,16 @@ struct _xsltStackElem {
     const xmlChar *name;	/* the local part of the name QName */
     const xmlChar *nameURI;	/* the URI part of the name QName */
     const xmlChar *select;	/* the eval string */
-    xmlNodePtr tree;		/* the tree if no eval string or the location */
+    xmlNodePtr tree;		/* the sequence constructor if no eval
+				    string or the location */
     xmlXPathObjectPtr value;	/* The value if computed */
+    xmlDocPtr fragment;		/* The Result Tree Fragments (needed for XSLT 1.0)
+				   which are bound to the variable's lifetime. */
+    int level;                  /* the depth in the tree;
+                                   -1 if persistent (e.g. a given xsl:with-param) */
+    xsltTransformContextPtr context; /* The transformation context; needed to cache
+                                        the variables */
+    int flags;
 };
 
 #ifdef XSLT_REFACTORED
@@ -1532,6 +1610,21 @@ struct _xsltStylesheet {
 #endif
 };
 
+typedef struct _xsltTransformCache xsltTransformCache;
+typedef xsltTransformCache *xsltTransformCachePtr;
+struct _xsltTransformCache {
+    xmlDocPtr RVT;
+    int nbRVT;
+    xsltStackElemPtr stackItems;
+    int nbStackItems;
+#ifdef XSLT_DEBUG_PROFILE_CACHE
+    int dbgCachedRVTs;
+    int dbgReusedRVTs;
+    int dbgCachedVars;
+    int dbgReusedVars;
+#endif
+};
+
 /*
  * The in-memory structure corresponding to an XSLT Transformation.
  */
@@ -1574,7 +1667,7 @@ struct _xsltTransformContext {
 
     xsltDocumentPtr docList;		/* the document list */
 
-    xsltDocumentPtr document;		/* the current document */
+    xsltDocumentPtr document;		/* the current source document; can be NULL if an RTF */
     xmlNodePtr node;			/* the current node being processed */
     xmlNodeSetPtr nodeList;		/* the current node list */
     /* xmlNodePtr current;			the node */
@@ -1618,6 +1711,7 @@ struct _xsltTransformContext {
 
     /*
      * handling of temporary Result Value Tree
+     * (XSLT 1.0 term: "Result Tree Fragment")
      */
     xmlDocPtr       tmpRVT;		/* list of RVT without persistance */
     xmlDocPtr       persistRVT;		/* list of persistant RVTs */
@@ -1641,17 +1735,22 @@ struct _xsltTransformContext {
      * dictionnary: shared between stylesheet, context and documents.
      */
     xmlDictPtr dict;
-    /*
-     * temporary storage for doc ptr, currently only used for
-     * global var evaluation
-     */
-    xmlDocPtr		tmpDoc;
+    xmlDocPtr		tmpDoc; /* Obsolete; not used in the library. */
     /*
      * all document text strings are internalized
      */
     int internalized;
     int nbKeys;
-    int hasTemplKeyPatterns;
+    int hasTemplKeyPatterns;    
+    xsltTemplatePtr currentTemplateRule; /* the Current Template Rule */    
+    xmlNodePtr initialContextNode;
+    xmlDocPtr initialContextDoc;
+    xsltTransformCachePtr cache;
+    void *contextVariable; /* the current variable item */
+    xmlDocPtr localRVT; /* list of local tree fragments; will be freed when
+			   the instruction which created the fragment
+                           exits */
+    xmlDocPtr localRVTBase;    
 };
 
 /**
@@ -1757,12 +1856,32 @@ XSLTPUBFUN xmlDocPtr XSLTCALL
 XSLTPUBFUN int XSLTCALL			
 			xsltRegisterTmpRVT	(xsltTransformContextPtr ctxt,
 						 xmlDocPtr RVT);
+XSLTPUBFUN int XSLTCALL			
+			xsltRegisterLocalRVT	(xsltTransformContextPtr ctxt,
+						 xmlDocPtr RVT);
 XSLTPUBFUN int XSLTCALL			
 			xsltRegisterPersistRVT	(xsltTransformContextPtr ctxt,
 						 xmlDocPtr RVT);
-XSLTPUBFUN void XSLTCALL			
+XSLTPUBFUN int XSLTCALL
+			xsltExtensionInstructionResultRegister(
+						 xsltTransformContextPtr ctxt,
+						 xmlXPathObjectPtr obj);
+XSLTPUBFUN int XSLTCALL
+			xsltExtensionInstructionResultFinalize(
+						 xsltTransformContextPtr ctxt);
+XSLTPUBFUN void XSLTCALL
 			xsltFreeRVTs		(xsltTransformContextPtr ctxt);
-			
+XSLTPUBFUN void XSLTCALL
+			xsltReleaseRVT		(xsltTransformContextPtr ctxt,
+						 xmlDocPtr RVT);
+XSLTPUBFUN int XSLTCALL
+			xsltTransStorageAdd	(xsltTransformContextPtr ctxt,
+						 void *id,
+						 void *data);
+XSLTPUBFUN void * XSLTCALL
+			xsltTransStorageRemove	(xsltTransformContextPtr ctxt,
+						 void *id);
+
 /*
  * Extra functions for Attribute Value Templates
  */
