@@ -26,19 +26,35 @@ libxslt_xsltParseStylesheetImportedDoc(PyObject *self ATTRIBUTE_UNUSED, PyObject
     xsltStylesheetPtr c_retval;
     xmlDocPtr doc;
     PyObject *pyobj_doc;
-    xsltStylesheetPtr style;
-    PyObject *pyobj_style;
+    xsltStylesheetPtr parentStyle;
+    PyObject *pyobj_parentStyle;
 
-    if (!PyArg_ParseTuple(args, (char *)"OO:xsltParseStylesheetImportedDoc", &pyobj_doc, &pyobj_style))
+    if (!PyArg_ParseTuple(args, (char *)"OO:xsltParseStylesheetImportedDoc", &pyobj_doc, &pyobj_parentStyle))
         return(NULL);
     doc = (xmlDocPtr) PyxmlNode_Get(pyobj_doc);
-    style = (xsltStylesheetPtr) Pystylesheet_Get(pyobj_style);
+    parentStyle = (xsltStylesheetPtr) Pystylesheet_Get(pyobj_parentStyle);
 
-    c_retval = xsltParseStylesheetImportedDoc(doc, style);
+    c_retval = xsltParseStylesheetImportedDoc(doc, parentStyle);
     py_retval = libxslt_xsltStylesheetPtrWrap((xsltStylesheetPtr) c_retval);
     return(py_retval);
 }
 
+PyObject *
+libxslt_xsltExtensionInstructionResultFinalize(PyObject *self ATTRIBUTE_UNUSED, PyObject *args) {
+    PyObject *py_retval;
+    int c_retval;
+    xsltTransformContextPtr ctxt;
+    PyObject *pyobj_ctxt;
+
+    if (!PyArg_ParseTuple(args, (char *)"O:xsltExtensionInstructionResultFinalize", &pyobj_ctxt))
+        return(NULL);
+    ctxt = (xsltTransformContextPtr) PytransformCtxt_Get(pyobj_ctxt);
+
+    c_retval = xsltExtensionInstructionResultFinalize(ctxt);
+    py_retval = libxml_intWrap((int) c_retval);
+    return(py_retval);
+}
+
 PyObject *
 libxslt_xsltGenerateIdFunction(PyObject *self ATTRIBUTE_UNUSED, PyObject *args) {
     xmlXPathParserContextPtr ctxt;
@@ -62,15 +78,15 @@ libxslt_xsltApplyAttributeSet(PyObject *self ATTRIBUTE_UNUSED, PyObject *args) {
     PyObject *pyobj_node;
     xmlNodePtr inst;
     PyObject *pyobj_inst;
-    xmlChar * attributes;
+    xmlChar * attrSets;
 
-    if (!PyArg_ParseTuple(args, (char *)"OOOz:xsltApplyAttributeSet", &pyobj_ctxt, &pyobj_node, &pyobj_inst, &attributes))
+    if (!PyArg_ParseTuple(args, (char *)"OOOz:xsltApplyAttributeSet", &pyobj_ctxt, &pyobj_node, &pyobj_inst, &attrSets))
         return(NULL);
     ctxt = (xsltTransformContextPtr) PytransformCtxt_Get(pyobj_ctxt);
     node = (xmlNodePtr) PyxmlNode_Get(pyobj_node);
     inst = (xmlNodePtr) PyxmlNode_Get(pyobj_inst);
 
-    xsltApplyAttributeSet(ctxt, node, inst, attributes);
+    xsltApplyAttributeSet(ctxt, node, inst, attrSets);
     Py_INCREF(Py_None);
     return(Py_None);
 }
@@ -83,20 +99,28 @@ libxslt_xsltAttrListTemplateProcess(PyObject *self ATTRIBUTE_UNUSED, PyObject *a
     PyObject *pyobj_ctxt;
     xmlNodePtr target;
     PyObject *pyobj_target;
-    xmlAttrPtr cur;
-    PyObject *pyobj_cur;
+    xmlAttrPtr attrs;
+    PyObject *pyobj_attrs;
 
-    if (!PyArg_ParseTuple(args, (char *)"OOO:xsltAttrListTemplateProcess", &pyobj_ctxt, &pyobj_target, &pyobj_cur))
+    if (!PyArg_ParseTuple(args, (char *)"OOO:xsltAttrListTemplateProcess", &pyobj_ctxt, &pyobj_target, &pyobj_attrs))
         return(NULL);
     ctxt = (xsltTransformContextPtr) PytransformCtxt_Get(pyobj_ctxt);
     target = (xmlNodePtr) PyxmlNode_Get(pyobj_target);
-    cur = (xmlAttrPtr) PyxmlNode_Get(pyobj_cur);
+    attrs = (xmlAttrPtr) PyxmlNode_Get(pyobj_attrs);
 
-    c_retval = xsltAttrListTemplateProcess(ctxt, target, cur);
+    c_retval = xsltAttrListTemplateProcess(ctxt, target, attrs);
     py_retval = libxml_xmlNodePtrWrap((xmlNodePtr) c_retval);
     return(py_retval);
 }
 
+PyObject *
+libxslt_xsltUninit(PyObject *self ATTRIBUTE_UNUSED, PyObject *args ATTRIBUTE_UNUSED) {
+
+    xsltUninit();
+    Py_INCREF(Py_None);
+    return(Py_None);
+}
+
 PyObject *
 libxslt_xsltParseStylesheetAttributeSet(PyObject *self ATTRIBUTE_UNUSED, PyObject *args) {
     xsltStylesheetPtr style;
@@ -197,7 +221,7 @@ libxslt_xsltGetDebuggerStatus(PyObject *self ATTRIBUTE_UNUSED, PyObject *args AT
 PyObject *
 libxslt_xsltStylesheetGetVersion(PyObject *self ATTRIBUTE_UNUSED, PyObject *args) {
     PyObject *py_retval;
-    xmlChar * c_retval;
+    const xmlChar * c_retval;
     xsltStylesheetPtr style;
     PyObject *pyobj_style;
 
@@ -206,7 +230,7 @@ libxslt_xsltStylesheetGetVersion(PyObject *self ATTRIBUTE_UNUSED, PyObject *args
     style = (xsltStylesheetPtr) Pystylesheet_Get(pyobj_style);
 
     c_retval = style->version;
-    py_retval = libxml_xmlCharPtrWrap((xmlChar *) c_retval);
+    py_retval = libxml_xmlCharPtrConstWrap((const xmlChar *) c_retval);
     return(py_retval);
 }
 
@@ -275,6 +299,23 @@ libxslt_xsltNextImport(PyObject *self ATTRIBUTE_UNUSED, PyObject *args) {
     return(py_retval);
 }
 
+PyObject *
+libxslt_xsltReleaseRVT(PyObject *self ATTRIBUTE_UNUSED, PyObject *args) {
+    xsltTransformContextPtr ctxt;
+    PyObject *pyobj_ctxt;
+    xmlDocPtr RVT;
+    PyObject *pyobj_RVT;
+
+    if (!PyArg_ParseTuple(args, (char *)"OO:xsltReleaseRVT", &pyobj_ctxt, &pyobj_RVT))
+        return(NULL);
+    ctxt = (xsltTransformContextPtr) PytransformCtxt_Get(pyobj_ctxt);
+    RVT = (xmlDocPtr) PyxmlNode_Get(pyobj_RVT);
+
+    xsltReleaseRVT(ctxt, RVT);
+    Py_INCREF(Py_None);
+    return(Py_None);
+}
+
 PyObject *
 libxslt_xsltFreeCtxtExts(PyObject *self ATTRIBUTE_UNUSED, PyObject *args) {
     xsltTransformContextPtr ctxt;
@@ -306,22 +347,6 @@ libxslt_xsltNamespaceAlias(PyObject *self ATTRIBUTE_UNUSED, PyObject *args) {
     return(Py_None);
 }
 
-PyObject *
-libxslt_xsltTransformGetModeURI(PyObject *self ATTRIBUTE_UNUSED, PyObject *args) {
-    PyObject *py_retval;
-    const xmlChar * c_retval;
-    xsltTransformContextPtr ctxt;
-    PyObject *pyobj_ctxt;
-
-    if (!PyArg_ParseTuple(args, (char *)"O:xsltTransformGetModeURI", &pyobj_ctxt))
-        return(NULL);
-    ctxt = (xsltTransformContextPtr) PytransformCtxt_Get(pyobj_ctxt);
-
-    c_retval = ctxt->modeURI;
-    py_retval = libxml_xmlCharPtrConstWrap((const xmlChar *) c_retval);
-    return(py_retval);
-}
-
 PyObject *
 libxslt_xsltFreeDocuments(PyObject *self ATTRIBUTE_UNUSED, PyObject *args) {
     xsltTransformContextPtr ctxt;
@@ -597,18 +622,18 @@ libxslt_xsltCopyNamespace(PyObject *self ATTRIBUTE_UNUSED, PyObject *args) {
     xmlNsPtr c_retval;
     xsltTransformContextPtr ctxt;
     PyObject *pyobj_ctxt;
-    xmlNodePtr node;
-    PyObject *pyobj_node;
-    xmlNsPtr cur;
-    PyObject *pyobj_cur;
+    xmlNodePtr elem;
+    PyObject *pyobj_elem;
+    xmlNsPtr ns;
+    PyObject *pyobj_ns;
 
-    if (!PyArg_ParseTuple(args, (char *)"OOO:xsltCopyNamespace", &pyobj_ctxt, &pyobj_node, &pyobj_cur))
+    if (!PyArg_ParseTuple(args, (char *)"OOO:xsltCopyNamespace", &pyobj_ctxt, &pyobj_elem, &pyobj_ns))
         return(NULL);
     ctxt = (xsltTransformContextPtr) PytransformCtxt_Get(pyobj_ctxt);
-    node = (xmlNodePtr) PyxmlNode_Get(pyobj_node);
-    cur = (xmlNsPtr) PyxmlNode_Get(pyobj_cur);
+    elem = (xmlNodePtr) PyxmlNode_Get(pyobj_elem);
+    ns = (xmlNsPtr) PyxmlNode_Get(pyobj_ns);
 
-    c_retval = xsltCopyNamespace(ctxt, node, cur);
+    c_retval = xsltCopyNamespace(ctxt, elem, ns);
     py_retval = libxml_xmlNsPtrWrap((xmlNsPtr) c_retval);
     return(py_retval);
 }
@@ -617,7 +642,7 @@ PyObject *
 libxslt_xsltCalibrateAdjust(PyObject *self ATTRIBUTE_UNUSED, PyObject *args) {
     long delta;
 
-    if (!PyArg_ParseTuple(args, (char *)"i:xsltCalibrateAdjust", &delta))
+    if (!PyArg_ParseTuple(args, (char *)"l:xsltCalibrateAdjust", &delta))
         return(NULL);
 
     xsltCalibrateAdjust(delta);
@@ -645,7 +670,7 @@ libxslt_xsltSaveProfiling(PyObject *self ATTRIBUTE_UNUSED, PyObject *args) {
 PyObject *
 libxslt_xsltStylesheetGetMethod(PyObject *self ATTRIBUTE_UNUSED, PyObject *args) {
     PyObject *py_retval;
-    xmlChar * c_retval;
+    const xmlChar * c_retval;
     xsltStylesheetPtr style;
     PyObject *pyobj_style;
 
@@ -654,7 +679,7 @@ libxslt_xsltStylesheetGetMethod(PyObject *self ATTRIBUTE_UNUSED, PyObject *args)
     style = (xsltStylesheetPtr) Pystylesheet_Get(pyobj_style);
 
     c_retval = style->method;
-    py_retval = libxml_xmlCharPtrWrap((xmlChar *) c_retval);
+    py_retval = libxml_xmlCharPtrConstWrap((const xmlChar *) c_retval);
     return(py_retval);
 }
 
@@ -693,13 +718,13 @@ libxslt_xsltCheckExtPrefix(PyObject *self ATTRIBUTE_UNUSED, PyObject *args) {
     int c_retval;
     xsltStylesheetPtr style;
     PyObject *pyobj_style;
-    xmlChar * prefix;
+    xmlChar * URI;
 
-    if (!PyArg_ParseTuple(args, (char *)"Oz:xsltCheckExtPrefix", &pyobj_style, &prefix))
+    if (!PyArg_ParseTuple(args, (char *)"Oz:xsltCheckExtPrefix", &pyobj_style, &URI))
         return(NULL);
     style = (xsltStylesheetPtr) Pystylesheet_Get(pyobj_style);
 
-    c_retval = xsltCheckExtPrefix(style, prefix);
+    c_retval = xsltCheckExtPrefix(style, URI);
     py_retval = libxml_intWrap((int) c_retval);
     return(py_retval);
 }
@@ -783,18 +808,18 @@ libxslt_xsltEvalTemplateString(PyObject *self ATTRIBUTE_UNUSED, PyObject *args)
     xmlChar * c_retval;
     xsltTransformContextPtr ctxt;
     PyObject *pyobj_ctxt;
-    xmlNodePtr node;
-    PyObject *pyobj_node;
-    xmlNodePtr parent;
-    PyObject *pyobj_parent;
+    xmlNodePtr contextNode;
+    PyObject *pyobj_contextNode;
+    xmlNodePtr inst;
+    PyObject *pyobj_inst;
 
-    if (!PyArg_ParseTuple(args, (char *)"OOO:xsltEvalTemplateString", &pyobj_ctxt, &pyobj_node, &pyobj_parent))
+    if (!PyArg_ParseTuple(args, (char *)"OOO:xsltEvalTemplateString", &pyobj_ctxt, &pyobj_contextNode, &pyobj_inst))
         return(NULL);
     ctxt = (xsltTransformContextPtr) PytransformCtxt_Get(pyobj_ctxt);
-    node = (xmlNodePtr) PyxmlNode_Get(pyobj_node);
-    parent = (xmlNodePtr) PyxmlNode_Get(pyobj_parent);
+    contextNode = (xmlNodePtr) PyxmlNode_Get(pyobj_contextNode);
+    inst = (xmlNodePtr) PyxmlNode_Get(pyobj_inst);
 
-    c_retval = xsltEvalTemplateString(ctxt, node, parent);
+    c_retval = xsltEvalTemplateString(ctxt, contextNode, inst);
     py_retval = libxml_xmlCharPtrWrap((xmlChar *) c_retval);
     return(py_retval);
 }
@@ -861,34 +886,19 @@ libxslt_xsltAttrTemplateValueProcessNode(PyObject *self ATTRIBUTE_UNUSED, PyObje
     xsltTransformContextPtr ctxt;
     PyObject *pyobj_ctxt;
     xmlChar * str;
-    xmlNodePtr node;
-    PyObject *pyobj_node;
+    xmlNodePtr inst;
+    PyObject *pyobj_inst;
 
-    if (!PyArg_ParseTuple(args, (char *)"OzO:xsltAttrTemplateValueProcessNode", &pyobj_ctxt, &str, &pyobj_node))
+    if (!PyArg_ParseTuple(args, (char *)"OzO:xsltAttrTemplateValueProcessNode", &pyobj_ctxt, &str, &pyobj_inst))
         return(NULL);
     ctxt = (xsltTransformContextPtr) PytransformCtxt_Get(pyobj_ctxt);
-    node = (xmlNodePtr) PyxmlNode_Get(pyobj_node);
+    inst = (xmlNodePtr) PyxmlNode_Get(pyobj_inst);
 
-    c_retval = xsltAttrTemplateValueProcessNode(ctxt, str, node);
+    c_retval = xsltAttrTemplateValueProcessNode(ctxt, str, inst);
     py_retval = libxml_xmlCharPtrWrap((xmlChar *) c_retval);
     return(py_retval);
 }
 
-PyObject *
-libxslt_xsltSystemPropertyFunction(PyObject *self ATTRIBUTE_UNUSED, PyObject *args) {
-    xmlXPathParserContextPtr ctxt;
-    PyObject *pyobj_ctxt;
-    int nargs;
-
-    if (!PyArg_ParseTuple(args, (char *)"Oi:xsltSystemPropertyFunction", &pyobj_ctxt, &nargs))
-        return(NULL);
-    ctxt = (xmlXPathParserContextPtr) PyxmlXPathParserContext_Get(pyobj_ctxt);
-
-    xsltSystemPropertyFunction(ctxt, nargs);
-    Py_INCREF(Py_None);
-    return(Py_None);
-}
-
 PyObject *
 libxslt_xsltUnregisterExtModuleTopLevel(PyObject *self ATTRIBUTE_UNUSED, PyObject *args) {
     PyObject *py_retval;
@@ -997,15 +1007,15 @@ PyObject *
 libxslt_xsltParseStylesheetVariable(PyObject *self ATTRIBUTE_UNUSED, PyObject *args) {
     xsltTransformContextPtr ctxt;
     PyObject *pyobj_ctxt;
-    xmlNodePtr cur;
-    PyObject *pyobj_cur;
+    xmlNodePtr inst;
+    PyObject *pyobj_inst;
 
-    if (!PyArg_ParseTuple(args, (char *)"OO:xsltParseStylesheetVariable", &pyobj_ctxt, &pyobj_cur))
+    if (!PyArg_ParseTuple(args, (char *)"OO:xsltParseStylesheetVariable", &pyobj_ctxt, &pyobj_inst))
         return(NULL);
     ctxt = (xsltTransformContextPtr) PytransformCtxt_Get(pyobj_ctxt);
-    cur = (xmlNodePtr) PyxmlNode_Get(pyobj_cur);
+    inst = (xmlNodePtr) PyxmlNode_Get(pyobj_inst);
 
-    xsltParseStylesheetVariable(ctxt, cur);
+    xsltParseStylesheetVariable(ctxt, inst);
     Py_INCREF(Py_None);
     return(Py_None);
 }
@@ -1016,20 +1026,20 @@ libxslt_xsltGetSpecialNamespace(PyObject *self ATTRIBUTE_UNUSED, PyObject *args)
     xmlNsPtr c_retval;
     xsltTransformContextPtr ctxt;
     PyObject *pyobj_ctxt;
-    xmlNodePtr cur;
-    PyObject *pyobj_cur;
-    xmlChar * URI;
-    xmlChar * prefix;
-    xmlNodePtr out;
-    PyObject *pyobj_out;
+    xmlNodePtr invocNode;
+    PyObject *pyobj_invocNode;
+    xmlChar * nsName;
+    xmlChar * nsPrefix;
+    xmlNodePtr target;
+    PyObject *pyobj_target;
 
-    if (!PyArg_ParseTuple(args, (char *)"OOzzO:xsltGetSpecialNamespace", &pyobj_ctxt, &pyobj_cur, &URI, &prefix, &pyobj_out))
+    if (!PyArg_ParseTuple(args, (char *)"OOzzO:xsltGetSpecialNamespace", &pyobj_ctxt, &pyobj_invocNode, &nsName, &nsPrefix, &pyobj_target))
         return(NULL);
     ctxt = (xsltTransformContextPtr) PytransformCtxt_Get(pyobj_ctxt);
-    cur = (xmlNodePtr) PyxmlNode_Get(pyobj_cur);
-    out = (xmlNodePtr) PyxmlNode_Get(pyobj_out);
+    invocNode = (xmlNodePtr) PyxmlNode_Get(pyobj_invocNode);
+    target = (xmlNodePtr) PyxmlNode_Get(pyobj_target);
 
-    c_retval = xsltGetSpecialNamespace(ctxt, cur, URI, prefix, out);
+    c_retval = xsltGetSpecialNamespace(ctxt, invocNode, nsName, nsPrefix, target);
     py_retval = libxml_xmlNsPtrWrap((xmlNsPtr) c_retval);
     return(py_retval);
 }
@@ -1090,16 +1100,16 @@ libxslt_xsltAttrTemplateProcess(PyObject *self ATTRIBUTE_UNUSED, PyObject *args)
     PyObject *pyobj_ctxt;
     xmlNodePtr target;
     PyObject *pyobj_target;
-    xmlAttrPtr cur;
-    PyObject *pyobj_cur;
+    xmlAttrPtr attr;
+    PyObject *pyobj_attr;
 
-    if (!PyArg_ParseTuple(args, (char *)"OOO:xsltAttrTemplateProcess", &pyobj_ctxt, &pyobj_target, &pyobj_cur))
+    if (!PyArg_ParseTuple(args, (char *)"OOO:xsltAttrTemplateProcess", &pyobj_ctxt, &pyobj_target, &pyobj_attr))
         return(NULL);
     ctxt = (xsltTransformContextPtr) PytransformCtxt_Get(pyobj_ctxt);
     target = (xmlNodePtr) PyxmlNode_Get(pyobj_target);
-    cur = (xmlAttrPtr) PyxmlNode_Get(pyobj_cur);
+    attr = (xmlAttrPtr) PyxmlNode_Get(pyobj_attr);
 
-    c_retval = xsltAttrTemplateProcess(ctxt, target, cur);
+    c_retval = xsltAttrTemplateProcess(ctxt, target, attr);
     py_retval = libxml_xmlNodePtrWrap((xmlNodePtr) c_retval);
     return(py_retval);
 }
@@ -1136,6 +1146,14 @@ libxslt_xsltTransformGetOutputDoc(PyObject *self ATTRIBUTE_UNUSED, PyObject *arg
     return(py_retval);
 }
 
+PyObject *
+libxslt_xsltInit(PyObject *self ATTRIBUTE_UNUSED, PyObject *args ATTRIBUTE_UNUSED) {
+
+    xsltInit();
+    Py_INCREF(Py_None);
+    return(Py_None);
+}
+
 PyObject *
 libxslt_xsltInitCtxtExts(PyObject *self ATTRIBUTE_UNUSED, PyObject *args) {
     PyObject *py_retval;
@@ -1155,7 +1173,7 @@ libxslt_xsltInitCtxtExts(PyObject *self ATTRIBUTE_UNUSED, PyObject *args) {
 PyObject *
 libxslt_xsltStylesheetGetEncoding(PyObject *self ATTRIBUTE_UNUSED, PyObject *args) {
     PyObject *py_retval;
-    xmlChar * c_retval;
+    const xmlChar * c_retval;
     xsltStylesheetPtr style;
     PyObject *pyobj_style;
 
@@ -1164,7 +1182,7 @@ libxslt_xsltStylesheetGetEncoding(PyObject *self ATTRIBUTE_UNUSED, PyObject *arg
     style = (xsltStylesheetPtr) Pystylesheet_Get(pyobj_style);
 
     c_retval = style->encoding;
-    py_retval = libxml_xmlCharPtrWrap((xmlChar *) c_retval);
+    py_retval = libxml_xmlCharPtrConstWrap((const xmlChar *) c_retval);
     return(py_retval);
 }
 
@@ -1293,20 +1311,6 @@ libxslt_xsltFreeNamespaceAliasHashes(PyObject *self ATTRIBUTE_UNUSED, PyObject *
     return(Py_None);
 }
 
-PyObject *
-libxslt_xsltShutdownCtxtExts(PyObject *self ATTRIBUTE_UNUSED, PyObject *args) {
-    xsltTransformContextPtr ctxt;
-    PyObject *pyobj_ctxt;
-
-    if (!PyArg_ParseTuple(args, (char *)"O:xsltShutdownCtxtExts", &pyobj_ctxt))
-        return(NULL);
-    ctxt = (xsltTransformContextPtr) PytransformCtxt_Get(pyobj_ctxt);
-
-    xsltShutdownCtxtExts(ctxt);
-    Py_INCREF(Py_None);
-    return(Py_None);
-}
-
 PyObject *
 libxslt_xsltGetPlainNamespace(PyObject *self ATTRIBUTE_UNUSED, PyObject *args) {
     PyObject *py_retval;
@@ -1417,7 +1421,7 @@ libxslt_xsltTimestamp(PyObject *self ATTRIBUTE_UNUSED, PyObject *args ATTRIBUTE_
     long c_retval;
 
     c_retval = xsltTimestamp();
-    py_retval = libxml_intWrap((int) c_retval);
+    py_retval = libxml_longWrap((long) c_retval);
     return(py_retval);
 }
 
@@ -1586,21 +1590,35 @@ libxslt_xsltEvalAttrValueTemplate(PyObject *self ATTRIBUTE_UNUSED, PyObject *arg
     xmlChar * c_retval;
     xsltTransformContextPtr ctxt;
     PyObject *pyobj_ctxt;
-    xmlNodePtr node;
-    PyObject *pyobj_node;
+    xmlNodePtr inst;
+    PyObject *pyobj_inst;
     xmlChar * name;
     xmlChar * ns;
 
-    if (!PyArg_ParseTuple(args, (char *)"OOzz:xsltEvalAttrValueTemplate", &pyobj_ctxt, &pyobj_node, &name, &ns))
+    if (!PyArg_ParseTuple(args, (char *)"OOzz:xsltEvalAttrValueTemplate", &pyobj_ctxt, &pyobj_inst, &name, &ns))
         return(NULL);
     ctxt = (xsltTransformContextPtr) PytransformCtxt_Get(pyobj_ctxt);
-    node = (xmlNodePtr) PyxmlNode_Get(pyobj_node);
+    inst = (xmlNodePtr) PyxmlNode_Get(pyobj_inst);
 
-    c_retval = xsltEvalAttrValueTemplate(ctxt, node, name, ns);
+    c_retval = xsltEvalAttrValueTemplate(ctxt, inst, name, ns);
     py_retval = libxml_xmlCharPtrWrap((xmlChar *) c_retval);
     return(py_retval);
 }
 
+PyObject *
+libxslt_xsltShutdownCtxtExts(PyObject *self ATTRIBUTE_UNUSED, PyObject *args) {
+    xsltTransformContextPtr ctxt;
+    PyObject *pyobj_ctxt;
+
+    if (!PyArg_ParseTuple(args, (char *)"O:xsltShutdownCtxtExts", &pyobj_ctxt))
+        return(NULL);
+    ctxt = (xsltTransformContextPtr) PytransformCtxt_Get(pyobj_ctxt);
+
+    xsltShutdownCtxtExts(ctxt);
+    Py_INCREF(Py_None);
+    return(Py_None);
+}
+
 PyObject *
 libxslt_xsltTransformGetCurrent(PyObject *self ATTRIBUTE_UNUSED, PyObject *args) {
     PyObject *py_retval;
@@ -1693,6 +1711,21 @@ libxslt_xsltGetCNsProp(PyObject *self ATTRIBUTE_UNUSED, PyObject *args) {
     return(py_retval);
 }
 
+PyObject *
+libxslt_xsltSystemPropertyFunction(PyObject *self ATTRIBUTE_UNUSED, PyObject *args) {
+    xmlXPathParserContextPtr ctxt;
+    PyObject *pyobj_ctxt;
+    int nargs;
+
+    if (!PyArg_ParseTuple(args, (char *)"Oi:xsltSystemPropertyFunction", &pyobj_ctxt, &nargs))
+        return(NULL);
+    ctxt = (xmlXPathParserContextPtr) PyxmlXPathParserContext_Get(pyobj_ctxt);
+
+    xsltSystemPropertyFunction(ctxt, nargs);
+    Py_INCREF(Py_None);
+    return(Py_None);
+}
+
 PyObject *
 libxslt_xsltDebugDumpExtensions(PyObject *self ATTRIBUTE_UNUSED, PyObject *args) {
     FILE * output;
@@ -1726,14 +1759,14 @@ libxslt_xsltStylePreCompute(PyObject *self ATTRIBUTE_UNUSED, PyObject *args) {
 
 PyObject *
 libxslt_xsltFreeStylesheet(PyObject *self ATTRIBUTE_UNUSED, PyObject *args) {
-    xsltStylesheetPtr sheet;
-    PyObject *pyobj_sheet;
+    xsltStylesheetPtr style;
+    PyObject *pyobj_style;
 
-    if (!PyArg_ParseTuple(args, (char *)"O:xsltFreeStylesheet", &pyobj_sheet))
+    if (!PyArg_ParseTuple(args, (char *)"O:xsltFreeStylesheet", &pyobj_style))
         return(NULL);
-    sheet = (xsltStylesheetPtr) Pystylesheet_Get(pyobj_sheet);
+    style = (xsltStylesheetPtr) Pystylesheet_Get(pyobj_style);
 
-    xsltFreeStylesheet(sheet);
+    xsltFreeStylesheet(style);
     Py_INCREF(Py_None);
     return(Py_None);
 }
@@ -1863,7 +1896,7 @@ libxslt_xsltDebug(PyObject *self ATTRIBUTE_UNUSED, PyObject *args) {
 PyObject *
 libxslt_xsltStylesheetGetMethodURI(PyObject *self ATTRIBUTE_UNUSED, PyObject *args) {
     PyObject *py_retval;
-    xmlChar * c_retval;
+    const xmlChar * c_retval;
     xsltStylesheetPtr style;
     PyObject *pyobj_style;
 
@@ -1872,7 +1905,26 @@ libxslt_xsltStylesheetGetMethodURI(PyObject *self ATTRIBUTE_UNUSED, PyObject *ar
     style = (xsltStylesheetPtr) Pystylesheet_Get(pyobj_style);
 
     c_retval = style->methodURI;
-    py_retval = libxml_xmlCharPtrWrap((xmlChar *) c_retval);
+    py_retval = libxml_xmlCharPtrConstWrap((const xmlChar *) c_retval);
+    return(py_retval);
+}
+
+PyObject *
+libxslt_xsltRegisterLocalRVT(PyObject *self ATTRIBUTE_UNUSED, PyObject *args) {
+    PyObject *py_retval;
+    int c_retval;
+    xsltTransformContextPtr ctxt;
+    PyObject *pyobj_ctxt;
+    xmlDocPtr RVT;
+    PyObject *pyobj_RVT;
+
+    if (!PyArg_ParseTuple(args, (char *)"OO:xsltRegisterLocalRVT", &pyobj_ctxt, &pyobj_RVT))
+        return(NULL);
+    ctxt = (xsltTransformContextPtr) PytransformCtxt_Get(pyobj_ctxt);
+    RVT = (xmlDocPtr) PyxmlNode_Get(pyobj_RVT);
+
+    c_retval = xsltRegisterLocalRVT(ctxt, RVT);
+    py_retval = libxml_intWrap((int) c_retval);
     return(py_retval);
 }
 
@@ -1907,6 +1959,22 @@ libxslt_xsltKeyFunction(PyObject *self ATTRIBUTE_UNUSED, PyObject *args) {
     return(Py_None);
 }
 
+PyObject *
+libxslt_xsltTransformGetModeURI(PyObject *self ATTRIBUTE_UNUSED, PyObject *args) {
+    PyObject *py_retval;
+    const xmlChar * c_retval;
+    xsltTransformContextPtr ctxt;
+    PyObject *pyobj_ctxt;
+
+    if (!PyArg_ParseTuple(args, (char *)"O:xsltTransformGetModeURI", &pyobj_ctxt))
+        return(NULL);
+    ctxt = (xsltTransformContextPtr) PytransformCtxt_Get(pyobj_ctxt);
+
+    c_retval = ctxt->modeURI;
+    py_retval = libxml_xmlCharPtrConstWrap((const xmlChar *) c_retval);
+    return(py_retval);
+}
+
 PyObject *
 libxslt_xsltRegisterExtPrefix(PyObject *self ATTRIBUTE_UNUSED, PyObject *args) {
     PyObject *py_retval;
